@@ -1,5 +1,6 @@
 import math
 import numpy
+import copy
 from .db import CorpsEval
 from krx_hj3415 import krx
 from util_hj3415 import utils
@@ -109,12 +110,13 @@ class Eval:
 
     def mil(self) -> dict:
         def calcPFCF(fcf: dict) -> dict:
-            for k, v in fcf.items():
+            pfcf = copy.deepcopy(fcf)
+            for k, v in pfcf.items():
                 if v == 0:
                     continue
                 else:
-                    fcf[k] = round(self.marketcap / v, 2)
-            return fcf
+                    pfcf[k] = round(self.marketcap / v, 2)
+            return pfcf
 
         지배주주당기순이익 = self.corp_eval_db.calc당기순이익()
         self.corp_eval_db.set_col('c104q')
@@ -208,7 +210,7 @@ class Score:
         self.code = code
         self.eval_dict = Eval(code).get_all()
 
-    def red(self) -> int:
+    def red(self) -> tuple:
         """
         red price와 최근 주가의 괴리율 파악
         양수면 주가가 고평가되어 있는 상태, 음수면 저평가
@@ -231,13 +233,13 @@ class Score:
         logger.info(f'괴리율 : {괴리율}')
         p = 0
         if math.isnan(괴리율) or red_price <= 0:
-            pass
+            return p, None
         else:
             p += -math.ceil(괴리율 / 10)
         logger.info(f'point : {p}')
-        return p
+        return p, int(괴리율)
 
-    def mil(self) -> int:
+    def mil(self) -> tuple:
         """
         - 재무활동현금흐름이 마이너스라는 것은 배당급 지급했거나, 자사주 매입했거나, 부채를 상환한 상태임.
         - 반대는 채권자로 자금을 조달했거나 신주를 발행했다는 의미
@@ -275,46 +277,45 @@ class Score:
         p = 0
 
         # 주주수익률 평가
-        MAX = 5
+        MAX1 = 5
         주주수익률평가 = math.ceil(self.eval_dict['mil']['주주수익률'] - (Eval.EXPECT_EARN * 100))
-        주주수익률평가 = MAX if MAX < 주주수익률평가 else 주주수익률평가
+        주주수익률평가 = MAX1 if MAX1 < 주주수익률평가 else 주주수익률평가
         p += 주주수익률평가 if 0 < 주주수익률평가 else 0
 
         # 이익지표 평가
-        MAX = 2
-        p += MAX if self.eval_dict['mil']['이익지표'] < 0 else 0
+        MAX2 = 2
+        p += MAX2 if self.eval_dict['mil']['이익지표'] < 0 else 0
 
         # 투자수익률 평가
-        MAX = 3
+        MAX3 = 3
         roic = self.eval_dict['mil']['투자수익률']['ROIC']
         roe = self.eval_dict['mil']['투자수익률']['ROE']
         if roic is None or math.isnan(roic) or roic <= 0:
             if 10 < roe <= 20:
-                p += round(MAX * 0.333)
+                p += round(MAX3 * 0.333)
             elif 20 < roe:
-                p += round(MAX * 0.666)
+                p += round(MAX3 * 0.666)
         elif 0 < roic:
             if 0 < roic <= 15:
-                p += round(MAX * 0.333)
+                p += round(MAX3 * 0.333)
             elif 15 < roic <= 30:
-                p += round(MAX * 0.666)
+                p += round(MAX3 * 0.666)
             elif 30 < roic:
-                p += MAX
+                p += MAX3
 
         # PFCF 평가
-        MAX = 10
+        MAX4 = 10
         r_year, r_pfcf = Score.get_recent_from_c1034_dict(self.eval_dict['mil']['가치지표']['PFCF'])
         logger.info(f'recent pfcf {r_year}, {r_pfcf}')
         if r_pfcf is None or math.isnan(r_pfcf) or r_pfcf <= 0:
             pass
         else:
-            p += 0 if round(MAX - r_pfcf) < 0 else round(MAX - r_pfcf)
-        return p
+            p += 0 if round(MAX4 - r_pfcf) < 0 else round(MAX4 - r_pfcf)
+        return p, MAX1 + MAX2 + MAX3 + MAX4
 
     @staticmethod
     def get_recent_from_c1034_dict(d: dict) -> tuple:
         # pfcf 같이 데이터베이스의 딕셔너리가 아니라 계산된 딕셔너리를 위해서 만듬
-        import copy
         d2 = copy.deepcopy(d)
         try:
             del(d2['전분기대비'])
@@ -360,7 +361,7 @@ class Score:
         '영업이익률': {'동화약품': '10.23', '휴온스글로벌': '-74.99', '안트로젠': '6.18', '에스씨엠생명과학': '16.41', '이수앱지스': nan}}
         """
 
-        def 유동비율평가() -> int:
+        def 유동비율평가() -> tuple:
             # 채점은 0을 기준으로 마이너스 해간다. 즉 0이 제일 좋은 상태임.
             # 유동비율 평가 - 100 이하는 문제 있음
             p = 0
@@ -371,9 +372,9 @@ class Score:
             else:
                 p += 0 if 100 < round(유동비율) else NEG_MAX + round(유동비율/10)
             logger.info(f'point after 유동비율 : {p}')
-            return p
+            return p, NEG_MAX
 
-        def 이자보상배율평가() -> int:
+        def 이자보상배율평가() -> tuple:
             # 이자보상배율평가 - 1이면 자금사정빡빡 5이상이면 양호
             p = 0
             NEG_MAX = -5
@@ -394,9 +395,9 @@ class Score:
             else:
                 p += 0 if p == 0 else 1
             logger.info(f'point 이자보상배율 : {p}')
-            return p
+            return p, NEG_MAX
 
-        def 순운전자본회전율평가() -> int:
+        def 순운전자본회전율평가() -> tuple:
             # 순운전자본회전율은 매출액/순운전자본으로 일정비율이 유지되는것이 좋으며 너무 작아지면 순운전자본이 많아졌다는 의미로 재고나 외상이 쌓인다는 뜻
             p = 0
             NEG_MAX = -5
@@ -415,14 +416,14 @@ class Score:
             d_values = list(d.values())
             logger.info(f'd_values : {d_values}')
             if len(d_values) == 0:
-                return NEG_MAX
+                return NEG_MAX, NEG_MAX
             std = numpy.std(d_values)
             logger.info(f'표준편차 : {std}')
             p += NEG_MAX if round(std) > -NEG_MAX else -round(std)
             logger.info(f'point 순운전자본회전율 : {p}')
-            return p
+            return p, NEG_MAX
 
-        def 재고자산회전율평가() -> int:
+        def 재고자산회전율평가() -> tuple:
             # 재고자산회전율은 매출액/재고자산으로 회전율이 낮을수록 재고가 많다는 이야기이므로 불리 전년도등과 비교해서 큰차이 발생하면 알람.
             # 재고자산회전율이 작아지면 재고가 쌓인다는뜻
             p = 0
@@ -442,14 +443,14 @@ class Score:
             d_values = list(d.values())
             logger.info(f'd_values : {d_values}')
             if len(d_values) == 0:
-                return NEG_MAX
+                return NEG_MAX, NEG_MAX
             std = numpy.std(d_values)
             logger.info(f'표준편차 : {std}')
             p += NEG_MAX if round(std) > -NEG_MAX else -round(std)
             logger.info(f'point 재고자산회전율 : {p}')
-            return p
+            return p, NEG_MAX
 
-        def 순부채비율평가() -> int:
+        def 순부채비율평가() -> tuple:
             # 부채비율은 업종마다 달라 일괄비교 어려우나 순부채 비율이 20%이하인것이 좋고 꾸준히 늘어나지 않는것이 좋다.
             # 순부채 비율이 30%이상이면 좋치 않다.
 
@@ -475,9 +476,9 @@ class Score:
             else:
                 p += 0 if p == 0 else 1
             logger.info(f'point 순부채비율 : {p}')
-            return p
+            return p, NEG_MAX
 
-        def 매출액증가율평가():
+        def 매출액증가율평가() -> tuple:
             # 매출액은 어떤경우에도 성장하는 기업이 좋다.매출이 20%씩 늘어나는 종목은 유망한 종목
             p = 0
             매출액증가율 = self.eval_dict['blue']['매출액증가율']
@@ -496,17 +497,18 @@ class Score:
             if len(d_values) == 0:
                 return p
             mean = round(numpy.mean(d_values))
-            std = round(numpy.std(d_values))
-            logger.info(f'평균 : {mean}\t표준편차 : {std}')
-            p += 0 if (mean - std) < 0 else (mean - std)
+            #std = round(numpy.std(d_values))
+            logger.info(f'평균 : {mean}')
+            #logger.info(f'표준편차 : {std}')
+            p += 0 if mean < 0 else round(mean / 10)
             logger.info(f'point 매출액증가율 : {p}')
-            return p
+            return p, mean
 
-        def 영업이익률평가():
+        def 영업이익률평가() -> int:
             # 영업이익률은 기업의 경쟁력척도로 경쟁사에 비해 높으면 경제적해자를 갖춘셈
             p = 0
-            영업이익률 = self.eval_dict['blue']['영업이익률']
-            logger.info(f"영업이익률 : {self.eval_dict['blue']['영업이익률']}")
+            영업이익률 = copy.deepcopy(self.eval_dict['blue']['영업이익률'])
+            logger.info(f"영업이익률 : {영업이익률}")
 
             corp_eval_db = CorpsEval(self.code)
             corp_eval_db.set_col('c101')
@@ -530,11 +532,24 @@ class Score:
             logger.info(f'point 영업이익률 : {p}')
             return p
 
-        neg_p = 유동비율평가() + 이자보상배율평가() + 순운전자본회전율평가() + 재고자산회전율평가() + 순부채비율평가()
-        pos_p = 매출액증가율평가() + 영업이익률평가()
+        def calc_neg_total() -> tuple:
+            p1, m1 = 유동비율평가()
+            p2, m2 = 이자보상배율평가()
+            p3, m3 = 순운전자본회전율평가()
+            p4, m4 = 재고자산회전율평가()
+            p5, m5 = 순부채비율평가()
+            return p1 + p2 + p3 + p4 + p5, m1 + m2 + m3 + m4 + m5
 
-        logger.info(f'neg point : {neg_p}\t pos point : {pos_p}')
-        return neg_p, pos_p
+        def calc_pos_total() -> tuple:
+            p1, m = 매출액증가율평가()
+            p2 = 영업이익률평가()
+            return p1 + p2, m
+
+        neg_p, TOTAL_NEG_MAX = calc_neg_total()
+        pos_p, mean = calc_pos_total()
+
+        logger.info(f'neg point : {neg_p}/{TOTAL_NEG_MAX}\t pos point : {pos_p} 평균매출액증가율{mean}')
+        return neg_p, TOTAL_NEG_MAX, pos_p, mean
 
 
 def report_one(code: str) -> str:
@@ -561,47 +576,54 @@ def report_one(code: str) -> str:
                 f"발행주식: {utils.to_만(c101d['발행주식'])}주")
         return title + '\n' + intro + seperate_line + price + '\n' + info
 
-    def red_str(all_dict: dict, red_p: int) -> str:
+    def red_str(all_dict: dict, red_p: tuple) -> str:
+        red, 괴리율 = red_p
         red_dict = all_dict['red']
         logger.info(red_dict)
-        return (f"Red Price\tPoint({red_p})\n"
+        return (f"Red Price\tPoint({red})\n"
                 f"사업가치({deco_num(red_dict['사업가치'])}억) "
                 f"+ 재산가치({deco_num(red_dict['재산가치'])}억) "
                 f"- 부채({deco_num(red_dict['부채평가'])}억) "
                 f"/ 발행주식({utils.to_만(red_dict['발행주식수'])}주) "
                 f"= {deco_num(red_dict['red_price'])}원")
 
-    def mil_str(all_dict: dict, mil_p: int) -> str:
+    def mil_str(all_dict: dict, mil_p: tuple) -> str:
+        mil, MAX = mil_p
         mil_dict = all_dict['mil']
         logger.info(mil_dict)
-        return (f"Millenial\tPoint({mil_p})\n"
+        return (f"Millenial\tPoint({mil}/{MAX})\n"
                 f"1. 주주수익률: {mil_dict['주주수익률']} %\n"
                 f"2. 이익지표: {mil_dict['이익지표']}\n"
-                f"3. 투자수익률: 4분기합 ROIC: {mil_dict['투자수익률']['ROIC']} %, 최근 ROE: {mil_dict['투자수익률']['ROE']} %\n"
+                f"3. 투자수익률: ROIC 4분기합: {mil_dict['투자수익률']['ROIC']}%, 최근 ROE: {mil_dict['투자수익률']['ROE']}%\n"
                 f"4. 가치지표\n"
                 f"\tFCF: {mil_dict['가치지표']['FCF']}\n"
                 f"\tPFCF : {mil_dict['가치지표']['PFCF']}\n"
                 f"\tPCR: {mil_dict['가치지표']['PCR']}")
 
     def blue_str(all_dict: dict, blue_p: tuple) -> str:
-        neg_p, pos_p = blue_p
+        neg_p, NEG_MAX, pos_p, mean = blue_p
         blue_dict = all_dict['blue']
         logger.info(blue_dict)
-        return (f"Blue\t- Point({neg_p})\t+ Point({pos_p})\n"
+        return (f"Blue\t - Point({neg_p}/{NEG_MAX})\t+ Point({pos_p})\n"
                 f"1. 유동비율: {blue_dict['유동비율']}(100이하 위험)\n"
                 f"2. 이자보상배율: {blue_dict['이자보상배율']}(1이하 위험 5이상 양호)\n"
                 f"3. 순운전자본회전율: {blue_dict['순운전자본회전율']}\n"
                 f"4. 재고자산회전율: {blue_dict['재고자산회전율']}\n"
                 f"5. 순부채비율: {blue_dict['순부채비율']}(30이상 not good)\n"
                 f"6. 매출액증가율: {blue_dict['매출액증가율']}\n"
-                f"7. 영업이익률: {blue_dict['영업이익률']}\n")
+                f"\t평균매출액증가율: {mean}\n"
+                f"7. 영업이익률: {blue_dict['영업이익률']}")
 
     def c108_str(all_dict: dict) -> str:
         s = ''
-        for c108_dict in all_dict['c108']:
+        for i, c108_dict in enumerate(all_dict['c108']):
             logger.info(c108_dict)
             opinion = c108_dict['내용'].replace('▶', '\n\t')
-            s += f"\n{c108_dict['날짜']}\thprice : {c108_dict['목표가']} 원\n<<{c108_dict['제목']}>>{opinion}"
+            if i == 0:
+                pass
+            else:
+                s += '\n'
+            s += f"{c108_dict['날짜']}\thprice : {c108_dict['목표가']} 원\n<<{c108_dict['제목']}>>{opinion}"
         return s
 
     s = Score(code)
@@ -614,3 +636,82 @@ def report_one(code: str) -> str:
             + mil_str(d, s.mil()) + seperate_line
             + blue_str(d, s.blue()) + seperate_line
             + c108_str(d) + seperate_line)
+
+
+def telegram_one(code: str) -> str:
+    def deco_num(x: str) -> str:
+        return None if x is None or math.isnan(float(x)) else format(int(x), ",")
+
+    def c101_str(all_dict: dict) -> str:
+        c101d = all_dict['c101']
+        logger.info(c101d)
+        title = '=' * 35 + f"\t{c101d['코드']}\t\t{c101d['종목명']}\t\t{c101d['업종']}\t" + '=' * 35
+
+        intro = f"{c101d['intro']}"
+        price = (f"<< {c101d['date']} >>\n"
+                 f"주가: {deco_num(c101d['주가'])}원\n"
+                 f"52주최고: {deco_num(c101d['최고52주'])}원\n"
+                 f"52주최저: {deco_num(c101d['최저52주'])}원")
+        info = (f"PER: {c101d['PER']}\n"
+                f"업종PER: {c101d['업종PER']}\n"
+                f"PBR: {c101d['PBR']}\n"
+                f"배당수익률: {c101d['배당수익률']}%\n"
+                f"유통비율: {c101d['유통비율']}%\n"
+                f"거래대금: {utils.to_억(c101d['거래대금'])}원\n"
+                f"발행주식: {utils.to_만(c101d['발행주식'])}주\n"
+                f"시가총액: {deco_num(c101d['시가총액'])}억")
+        return title + '\n' + intro + seperate_line + price + '\n' + info
+
+    def red_str(all_dict: dict, red_p: tuple) -> str:
+        red, 괴리율 = red_p
+        red_dict = all_dict['red']
+        logger.info(red_dict)
+        return (f"<< Red Price >> Point({red})\n"
+                f"{deco_num(red_dict['red_price'])}원\n"
+                f"괴리율({괴리율}%)")
+
+    def mil_str(all_dict: dict, mil_p: tuple) -> str:
+        mil, MAX = mil_p
+        mil_dict = all_dict['mil']
+        logger.info(mil_dict)
+        return (f"<< Millenial >> Point({mil}/{MAX})\n"
+                f"1. 주주수익률: {mil_dict['주주수익률']} %\n"
+                f"2. 이익지표: {mil_dict['이익지표']}\n"
+                f"3. 투자수익률\nROIC 4분기합: {mil_dict['투자수익률']['ROIC']}%\n최근 ROE: {mil_dict['투자수익률']['ROE']}%\n"
+                f"4. 가치지표\n"
+                f"FCF\n{mil_dict['가치지표']['FCF']}\n"
+                f"PFCF\n{mil_dict['가치지표']['PFCF']}\n"
+                f"PCR\n{mil_dict['가치지표']['PCR']}")
+
+    def blue_str(all_dict: dict, blue_p: tuple) -> str:
+        neg_p, NEG_MAX, pos_p, mean = blue_p
+        blue_dict = all_dict['blue']
+        logger.info(blue_dict)
+        return (f"<< Blue >>\n"
+                f"Neg_P: {neg_p}/{NEG_MAX}\n"
+                f"Pos_P: {pos_p}\n"
+                f"평균매출액증가율: {mean}%\n"
+                f"영업이익률\n{blue_dict['영업이익률']}")
+
+    def c108_str(all_dict: dict) -> str:
+        s = ''
+        for i, c108_dict in enumerate(all_dict['c108']):
+            logger.info(c108_dict)
+            opinion = c108_dict['내용'].replace('▶', '\n\t')
+            if i == 0:
+                pass
+            else:
+                s += '\n'
+            s += f"{c108_dict['날짜']}\thprice : {c108_dict['목표가']} 원\n<<{c108_dict['제목']}>>{opinion}"
+        return s
+
+    s = Score(code)
+    d = s.eval_dict
+
+    seperate_line = '\n' + ('-' * 65) + '\n'
+
+    return (c101_str(d) + seperate_line
+            + red_str(d, s.red()) + seperate_line
+            + mil_str(d, s.mil()) + seperate_line
+            + blue_str(d, s.blue()) + seperate_line
+            + c108_str(d) + seperate_line)
